function [w, T, bias] = SCutFBRn(train_data, options, train_final)
    % SCutFBR.n in the paper.
    % It uses a two-level cross-validation to select fbr and threshold.
    % This improved FBR heuristic reset the threshold to the highest decision
    % value of the negative samples.

    % check input
    if (nargin ~= 3)
        if (nargin < 3)
            train_final = true;
        end
        if (nargin < 2)
            options = '-q -B 1';
        end
        if (nargin < 1 || nargin > 3)
            fprintf('Incorrect number of input parameters\n');
            return;
        end
    end
    
    x = train_data.x;
    y = train_data.y;
    map = train_data.map;

    % bias option
    bias = get_bias(options);
    if bias > 0
        w_size = size(x, 1) + 1;
    else
        w_size = size(x, 1);
    end
    
    t0all = clock;

    % number of label = number of model to be trained
    L = size(y, 2); 
    
    w = zeros(w_size, L);
    T = zeros(L, 1);

    % obtain (w,T) for each label
    % parpool(4);
    parfor i = 1:L
        fprintf(1, 'INFO: training %dth label (%d)...\n', i, map(i));
        t0 = clock;
        [w(:, i), T(i)] = train_one_label(2*y(:, i)-1, x, options, train_final);
        fprintf(1, '\tINFO: Done in %.6f seconds\n', etime(clock, t0));
    end

    fprintf(1, 'INFO: Total Time: %.6f seconds\n', etime(clock, t0all));
end

function [w, T] = train_one_label(proby, probx, options, train_final)
    % size(x) = (#label, #sample)

    fbr_list = [0.1 0.2 0.3 0.4 0.5 0.6 0.7 0.8];

    nr_fold = 3;

    bias = get_bias(options);

    warning('off', 'stats:cvpartition:KFoldMissingGrp');
    cv = cvpartition(proby, 'KFold', nr_fold, 'Stratify', true);

    f_list = zeros(size(fbr_list));

    % cv
    for fold = 1:nr_fold
        train_id = cv.training(fold);
        valid_id = cv.test(fold);

        train_y = proby(train_id);
        train_x = probx(:, train_id);
        valid_y = proby(valid_id);
        valid_x = probx(:, valid_id);

        % scutfbrn
        [scutfbr_w, scutfbr_b_list] = scutfbrn(train_y, train_x, fbr_list, options, true);
        if bias > 0
            valid_x = [valid_x; ones(1, size(valid_x, 2)) * bias];
        end
        wTx = valid_x' * scutfbr_w;
        
        F = fmeasure(valid_y, 2*(wTx > scutfbr_b_list')-1);
        f_list = f_list + F;
    end

    best_fbr = fbr_list(find(f_list == max(f_list), 1, 'last'));
    if max(f_list) == 0
        best_fbr = min(fbr_list);
        % fprintf(1, '\tINFO: train_one_label: F all 0\n');
    end

    % final model
    [w, T] = scutfbrn(proby, probx, best_fbr, options, train_final);
end



function [w, T_list] = scutfbrn(proby, probx, fbr_list, options, train_final)
    % given a list of fbr values, produce the threshold corresponding the the fbr value
    % through 1-level CV
    
    T_list = zeros(size(fbr_list, 2), 1);

    nr_fold = 3;

    bias = get_bias(options);
    
    warning('off', 'stats:cvpartition:KFoldMissingGrp');
    cv = cvpartition(proby, 'KFold', nr_fold, 'Stratify', true);

    % cv
    for fold = 1:nr_fold
        train_id = cv.training(fold);
        valid_id = cv.test(fold);

        train_y = proby(train_id);
        train_x = probx(:, train_id);
        valid_y = proby(valid_id);
        valid_x = probx(:, valid_id);

        % scut
        scut_w = do_train(train_y, train_x, options);
        if (bias > 0)
            valid_x = [valid_x; ones(1, size(valid_x, 2)) * bias];
        end
        wTx = valid_x' *  scut_w; % shape = (#sample in valid)
        scut_b = 0;

        start_F = fmeasure(valid_y, 2*(wTx > scut_b)-1);

        [sorted_wTx, sorted_wTx_index] = sort(wTx, 1, 'ascend'); 
        sorted_valid_y = valid_y(sorted_wTx_index);
        % best_F is initialized as predicting all as 1,
        % that is, the threshhold is set to the lowest
        tp = sum(valid_y == 1); fp = size(valid_y,1)-tp; fn = 0;
        cut = -1;
        best_F = (2*tp) / (2*tp + fp + fn);
        for i = 1:size(valid_y, 1)
            if sorted_valid_y(i) == -1,
                fp = fp - 1;
            else
                tp = tp - 1; fn = fn + 1;
            end
            F = (2*tp) / (2*tp + fp + fn);
            
            if F >= best_F
                best_F = F;
                cut = i;
            end
        end

        % modify b
        if best_F > start_F
            if cut == -1 % i.e., all +1
                val = sorted_wTx(1);
                scut_b = (val - eps(val)); % predict all +1
            elseif cut == size(valid_y, 1)
                val = sorted_wTx(size(valid_y, 1));
                scut_b = (val + eps(val));
            else
                scut_b = (sorted_wTx(cut) + sorted_wTx(cut+1))/2;
            end
        end

        F = fmeasure(valid_y, 2*(wTx > scut_b)-1);

        % compare to fbr_list
        for i = 1:size(fbr_list, 2)
            if F > fbr_list(i)
                T_list(i) = T_list(i) + scut_b;
            else
                T_list(i) = T_list(i) + max(wTx(valid_y < 0));
            end
        end
    end

    % final model
    T_list = T_list / nr_fold;
    if train_final
        w = do_train(proby, probx, options);
    else
        if bias > 0
            w_size = size(probx, 1) + 1;
        else
            w_size = size(probx, 1);
        end
        w = zeros(w_size, 1);
    end
end

