function [w, T, bias, best_a, best_b, a_list, b_list, cv_F] = smooth_all(train_data, options, train_final)
    % smooth-all (use CV to select a pair of a,b for all classes)
    % input:
    %   train_data: a struct with `y`,`x`,and `map` return by libsvmread_ml
    %   options: options to pass to LIBLINEAR
    %   train_final: whether to train the final linear model using all training data
    % output:
    %   w: weight of the linear SVMs. The shape is (#dim, #label)
    %   T: threshold for each label. The shape is (#label, 1)
    %   bias: the -B value passed to LIBLINEAR. -1 if not passed
    %   best_a, best_b: return the selected pari of (a,b).
    %   a_list, b_list, cv_F: return the list of (a,b) searched and their CV result.
    %       cv_F is of shape (#labels, #pairs)

    % check input
    if (nargin ~= 3)
        if nargin < 3
            train_final = true;
        end
        if nargin < 2
            options = '-q -B 1';
        end
        if nargin < 1 || nargin > 3
            fprintf('Incorrect number of input parameters\n');
            return;
        end
    end
    
    x = train_data.x;
    y = train_data.y;
    map = train_data.map;

    % bias option
    bias = get_bias(options);
    if bias > 0
        w_size = size(x, 1) + 1;
    else
        w_size = size(x, 1);
    end
        
    % number of label = number of model to be trained
    L = size(y, 2); 

    t0all = clock;

    % search over these values   
    bs = [0 1e-4 1e-3 1e-2 1e-1 1 10 1e2 1e3 1e4];
    
    N = size(y, 1);
    N_A = 30;
    a_list = [];
    for b = bs
        % use the range for p=1
        a = linspace(log(2*(b+1) / N), log(2*(b+1)), N_A);
        a_list = [a_list exp(a)];
    end
    b_list = repelem(bs, N_A);

    % sort a,b according to magnitude so that 
    % smaller a,b is selected when the CV result is the same
    apb = a_list + b_list;
    [~, arg] = sort(apb, 'ascend');
    reverse_arg(arg) = 1:numel(arg);
    a_list = a_list(arg);
    b_list = b_list(arg);

    % obtain CV result for each label
    cv_F = zeros(L, numel(a_list)); 
    % parpool(4);
    parfor i = 1:L
        fprintf(1, 'INFO: CV for %dth label (%d)...\n', i, map(i));
        t0 = clock;
        cv_F(i,:) = cv_one_label(2*y(:, i)-1, x, a_list, b_list, options);
        % fprintf(1, 'INFO: b = %g\n', b(i));
        fprintf(1, '\tINFO: Done in %.6f seconds\n', etime(clock, t0));
    end
    macro = mean(cv_F,1);

    best_idx = find(macro == max(macro), 1, 'first');
    best_a = a_list(best_idx);
    best_b = b_list(best_idx);
    if max(macro) == 0
        best_a = 1;
        best_b = 1;
    end

    w = zeros(w_size, L);
    T = zeros(L, 1);

    % train final w,T with the best param
    parfor i = 1:L
        fprintf(1, 'INFO: training %dth label (%d)...\n', i, map(i));
        t0 = clock;
        [w(:,i), T(i)] = train_for_ab(2*y(:, i)-1, x, best_a, best_b, options, train_final);
        fprintf(1, '\tINFO: Done in %.6f seconds\n', etime(clock, t0));
    end

    fprintf(1, 'INFO: Total Time: %.6f seconds\n', etime(clock, t0all));

    % un-sort the parameters
    a_list = a_list(reverse_arg);
    b_list = b_list(reverse_arg);
    cv_F = cv_F(:,reverse_arg);
end

function [f_list] = cv_one_label(proby, probx, a_list, b_list, options)
    % perform CV for the list of (a,b) and report the F-measure for each pair of (a,b)
    % inputs:
    %   proby: vector of size (#sample,1), elements are -1,+1
    %   probx: feature matrix of size (#feature, #sample)
    %   a_list: a list of parameter `a` to be tried
    %   b_list: a list of parameter `b` to be paired with a_list
    %           b_list should be of the same length as a_list
    %   options: options to be passed to LIBLINEAR

    nr_fold = 3;

    bias = get_bias(options);

    warning('off', 'stats:cvpartition:KFoldMissingGrp');
    cv = cvpartition(proby, 'KFold', nr_fold, 'Stratify', true);

    f_list = zeros(size(a_list));

    % cv
    for fold = 1:nr_fold
        train_id = cv.training(fold);
        valid_id = cv.test(fold);

        train_y = proby(train_id);
        train_x = probx(:, train_id);
        valid_y = proby(valid_id);
        valid_x = probx(:, valid_id);

        [w, T_list] = train_for_ab(train_y, train_x, a_list, b_list, options, true);
        if bias > 0
            valid_x = [valid_x; ones(1, size(valid_x, 2)) * bias];
        end
        wTx = valid_x' * w; % +b
        
        F = fmeasure(valid_y, 2*(wTx > T_list')-1);
        f_list = f_list + F;
    end

    f_list = f_list / nr_fold;
end






