function [w, T, bias, best_as, best_bs] = smooth_each(train_data, options, train_final)
    % smooth_each (select a pair of a,b for each label)
    % input:
    %   train_data: a struct with `y`,`x`,and `map` return by libsvmread_ml
    %   options: options to pass to LIBLINEAR
    %   train_final: whether to train the final linear model using all training data
    % output:
    %   w: weight of the linear SVMs. The shape is (#dim, #label)
    %   T: threshold for each label. The shape is (#label, 1)
    %   bias: the -B value passed to LIBLINEAR. -1 if not passed
    %   best_as, best_bs: return the selected a,b for each label. The shape is (#label,1) 
    
    % check input
    if (nargin ~= 3)
        if nargin < 3
            train_final = true;
        end
        if nargin < 2
            options = '-q -B 1';
        end
        if nargin < 1 || nargin > 3
            fprintf('Incorrect number of input parameters\n');
            return;
        end
    end

    x = train_data.x;
    y = train_data.y;
    map = train_data.map;

    % bias option
    bias = get_bias(options);
    if bias > 0
        w_size = size(x, 1) + 1;
    else
        w_size = size(x, 1);
    end
        
    t0all = clock;

    % number of label = number of model to be trained
    L = size(y, 2); 
    
    w = zeros(w_size, L);
    T = zeros(L, 1);
    best_as = zeros(L,1);
    best_bs = zeros(L,1);

    % obtain (w,T) for each label
    % parpool(4);
    parfor i = 1:L
        fprintf(1, 'INFO: training %dth label (%d)...\n', i, map(i));
        t0 = clock;
        [w(:, i), T(i), best_as(i), best_bs(i)] = train_one_label(2*y(:, i)-1, x, options, train_final);
        fprintf(1, '\tINFO: Done in %.6f seconds\n', etime(clock, t0));
    end
    fprintf(1, 'INFO: Total Time: %.6f seconds\n', etime(clock, t0all));
end

function [w, T, best_a, best_b] = train_one_label(proby, probx, options, train_final)
    % proby: vector of size (#sample,1), elements are -1,+1
    % probx: feature matrix of size (#feature, #sample)
    % options: options to be passed to LIBLINEAR
    % train_final: whether to train the final linear model using all data

    
    N = numel(proby);
    p = sum(proby > 0);
    n = N - p;

    bs = [0 1e-4 1e-3 1e-2 1e-1 1 10 1e2 1e3 1e4];
    a_list = [];
    b_list = [];
    for b = bs
        N_A = 30;
        a = linspace(log(2*(b+p) / N), log(2*(b+p)), N_A);
        b_list = [b_list repmat(b, 1, N_A)];
        a_list = [a_list exp(a)];
    end 
    % sort a,b according to magnitude so that 
    % smaller a,b is selected when the CV result is the same
    apb = a_list + b_list;
    [~, arg] = sort(apb, 'ascend');
    a_list = a_list(:,arg);
    b_list = b_list(:,arg);

    nr_fold = 3;

    bias = get_bias(options);

    warning('off', 'stats:cvpartition:KFoldMissingGrp');
    cv = cvpartition(proby, 'KFold', nr_fold, 'Stratify', true);

    f_list = zeros(size(a_list)); % result of F-measure from CV

    % cv
    for fold = 1:nr_fold
        train_id = cv.training(fold);
        valid_id = cv.test(fold);

        train_y = proby(train_id);
        train_x = probx(:, train_id);
        valid_y = proby(valid_id);
        valid_x = probx(:, valid_id);

        [w, T_list] = train_for_ab(train_y, train_x, a_list, b_list, options, true);
        if bias > 0
            valid_x = [valid_x; ones(1, size(valid_x, 2)) * bias];
        end
        wTx = valid_x' * w;
        
        F = fmeasure(valid_y, 2*(wTx > T_list')-1);
        f_list = f_list + F;
    end

    best_idx = find(f_list == max(f_list), 1, 'first');
    best_a = a_list(best_idx);
    best_b = b_list(best_idx);
    if max(f_list) == 0
        best_a = 1;
        best_b = 1;
    end

    % final model
    [w, T] = train_for_ab(proby, probx, best_a, best_b, options, train_final);
end







