function p_vs_f(data_name, model_name, options)
    % This function automatically searches for models 
    % with name `model_name` and option `options` 
    % train on dataset `data_name` in `./exp_outputs`.
    % Then, it outputs F-measure versus p (#positive sample) curve
    % averaged over these found runs.
    % It is only for binary relevance and SCut-based methods
    % example:
    %   data_name = 'blogcat'
    %   model_name = 'smooth-all'
    %   options = '-q -B 1'
    run('data_paths.m')

    output_dirs = {dir('.').name};
    dirs = {};
    for i = 1:length(output_dirs)
        f = output_dirs(i);
        if isdir(f) && contains(f, 'exp_outputs')
            new_dirs = fullfile(f, {dir(f{1}).name});
            dirs = cat(2, dirs, new_dirs);
        end
    end
    select = cellfun(@(x) contains(x, data_name), dirs) & cellfun(@isdir, dirs);
    dirs = {dirs{select}};

    if isempty(dirs)
        fprintf(2, "No experimental results found for %s on %s\n", model_name, data_name);
        return
    end

    fprintf(1, "Found experimental results:\n")
    dirs

    train_test = datasets(data_name);
    train_path = train_test{1};
    test_path = train_test{2};

    [y, x, map] = libsvmread_ml(train_path);
    [ty, tx, tmap] = libsvmread_ml(test_path);

    x = x';
    tx = tx';

    bias = get_bias(options);
    [x, tx] = fill_x(x, tx);
    if bias > 0
        tx=[tx; ones(1, size(tx,2)) * bias];
    end

    F = zeros(numel(map), 1);
    j = 1;
    for i = 1:numel(dirs)
        d = dirs{i};
        w_path = [d '/(' options ')/binary.mat'];
        T_path = [d '/(' options ')/' model_name '.mat'];

        if ~isfile(w_path)
            fprintf(2, 'ERROR: Cannot find the file for w\n');
            continue
        end
        if ~isfile(T_path)
            fprintf(2, 'ERROR: Cannot find the file for T\n');
            continue
        end
        disp(d)

        % load models
        w_file = matfile(w_path);
        w = w_file.w;
        if strcmp(model_name, 'binary')
            T = zeros(size(w,2),1);
        else
            T_file = matfile(T_path);
            T = T_file.T;
        end
        

        % predict and calculate F-measure
        pre = tx' * w - T';
        F(:,j) = fmeasures_per_label(ty, pre, tmap, map);
        j = j + 1;
        clear pre w T;
    end

    F = mean(F, 2);

    counts = sum(y,1); % p for each label
    density = 5;
    groups = round(log10(counts) * density) / density;

    group_counts = 10.^(unique(groups)); % p
    group_vals = grpstats(F, groups); % averaged F-measure

    table = array2table([group_counts' group_vals]);
    table.Properties.VariableNames(1:2) = {'p','F'};
    table
    writetable(table, ['./exp_outputs/p_vs_F/' data_name '_' model_name '_(' options ')_p-vs-F.csv']);
end